/**
 * Simple Javascript user interface to register a device with the Qnext network
 * Copyright Qnext Inc. 2002-2013
 * http://www.qnext.com
 * Date: 08/12/13
 * Time: 11:24 AM
 * Author Greg Wade (gregw@qnext.com)
 */

var HTTP_PORT = 4008;
var HTTPS_PORT = 4009;
var REDIRECT_SECONDS = 3;

var ACTIVATION_ID = "activation";
var CODE_FIELD_CLASS = "field-code";
var USERNAME_FIELD_CLASS = "field-user-name";
var PASSWORD_FIELD_CLASS = "field-user-password";
var DOMAIN_FIELD_CLASS = "field-user-domain";

var retryLink;

var startActivate = function () {

    psvFetchStatus();

    // enter a username into the edit box
    var username = gup("username");
    if (username != null && username.length > 0) {
        log("found username: " + username);
        getComp(USERNAME_FIELD_CLASS).val(username);
    }

    // enter a domain into the edit box
    var domain = gup("domain");
    if (domain != null && domain.length > 0) {
        log("found domain: " + domain);
        getComp(DOMAIN_FIELD_CLASS).val(domain);
    }

    // enter a code into the edit box
    var code = gup("code");
    if (code != null && code.length > 0) {
        log("found activation code: " + code);
        getComp(CODE_FIELD_CLASS).val(code);
    }

    retryLink = gup("retryLink");
    if (retryLink != null && retryLink.length > 0) {
        log("found retryLink: " + retryLink);
    } else {
        retryLink = null;
    }
};

function redirectUserEventually(url, seconds) {
    var msg = getLocaleStr("redirecting_secured") + " " + seconds + " " + getLocaleStr("redirecting_seconds");
    showWarning(msg);
    var remainingSeconds = seconds;
    var intervalId = setInterval(function () {
        remainingSeconds = redirectAtInterval(url, remainingSeconds, intervalId);
    }, 1000);
}

function redirectAtInterval(url, seconds, intervalId) {
    seconds = seconds - 1;
    var msg = getLocaleStr("redirecting_secured") + " " + seconds + " " + getLocaleStr("redirecting_seconds");
    log("redirecting to secured url in " + seconds + "s (interval id " + intervalId + ")");
    if (seconds <= 1) {
        log("clearing interval id " + intervalId);
        clearInterval(intervalId);
        log("performing redirection to " + url);
        window.location.replace(url); // don't save this page to history before redirect (FFS-4082)
    }
    showWarning(msg);
    return seconds;
}

// returns an object containing status:true/false
function validateCode(code, onFailure) {
    if (code.length < 4) {
        onFailure(getLocaleStr("error_code_short"));
        return false;
    }
    return true;
}

/**
 * Render the status in terms of a message, and highlighting errors
 *
 * @param {String} msg
 */
function showWarning(msg) {
    var $cont = $("#updatemsg");
    // remove previous errors using jquery - search for alerts? it will do
    $(".alert", $cont).remove();
    // add the error/success message and highlights
    $cont.append("<div class='alert alert-warning'>" + msg + "</div>");
}

/**
 * Request register a code
 *
 * @param {String} code
 * @param {String} userName
 * @param {String} userPassword
 * @param {String} userDomain
 * @param {String} [mspId]
 */
function psvRegisterCode(code, userName, userPassword, userDomain, mspId) {
    var params = {
        "cmd": "add",
        "code": code,
        "user": userName,
        "user-password": userPassword,
        "user-domain": userDomain
    };

    if (mspId)
        params["mspId"] = mspId;

    var jqXHR = $.ajax({
        method: "GET",
        url: "",
        data: params,
        dataType: "json"
    });

    jqXHR.done(function (resp) {
        if (resp.result) {
            handleActivationSuccess(resp.redirectURL);
            log("psvRegisterCode success with " + code);
        } else {
            activeButtonOnOff(false);
            translateErrorCode(resp.code);
            log("psvRegisterCode failure with " + code);
        }
    }).fail(function (xhr, err) {

        if (xhr.responseText) {
            var code;
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                code = error["code"];
                log("Code register failed error code: " + code);
            } catch (e) {
                log("Code register failed: " + e);
                code = ERROR_LOGIN;
            }
            log("Invalid login credentials: user=" + userName + " password=" + "xxx" + " domain=" + userDomain);
        } else {
            code = getError(xhr, err);
        }
        activeButtonOnOff(false);
        translateErrorCode(code);
    });
}

/**
 * Handle successful response
 *
 * @param {String} redirectURL
 */
function handleActivationSuccess(redirectURL) {
    // success
    log("activation successful");

    activeButtonOnOff(false);

    // indicate success visually
    showMessage(ERROR_OK, "activate_error_ok");
    getComp(CODE_FIELD_CLASS).val("");

    // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
    if (redirectURL != null) {
        // if there is a redirect URL, we immediately redirect the user
        log("redirectURL found: " + redirectURL);
        window.location.href = redirectURL;
    } else {
        log("no redirect URL was found");
    }

}

/**
 * function to fetch the status of the device
 * returns status info as JSON to the onSuccess function, in the format of:
 * {
 *    "version": "2.0.0",
 *    "build": "2.0.0",
 *    "updateable": false,
 *    "debug": true,
 *    "connected": false,
 *    "thumbnails": true
 * }
 *
 */
function psvFetchStatus() {
    if (TEST_LOCALLY) {
        setTimeout(function() {
            var status = {
                "version": "2.1.45",
                "build": "$Rev: 40606 $",
                "updateable": false,
                "debug": false,
                "os": "Linux",
                "platform": "Synology"
            };
            handleStatusSuccess(status);
        }, 1000);
    } else {
        var jqXHR = $.ajax({
            method: "GET",
            url: "",
            data: {
                "cmd": "status",
                "admin": "",
                "password": "",
                "user": ""
            },
            dataType: "json"
        });

        jqXHR.done(function (resp) {
            log("fetchStatus success with: ");
            log(resp);
            handleStatusSuccess(resp);
        }).fail(function (jqXHR, textStatus, ex) {
            log("fetchStatus failure status: " + jqXHR.status + " " + textStatus +
                (jqXHR.responseText ? ", response: " + jqXHR.responseText : "") +
                (ex ? ", exception: " + ex : ""));
        });
    }
}

/**
 * @param {Object} statusInfo format is:
 * {
 *    "version": "2.0.0",
 *    "build": "2.0.0",
 *    "updateable": false,
 *    "debug": true,
 *    "connected": false,
 *    "thumbnails": true
 * }
 */
function handleStatusSuccess(statusInfo) {

    // Set globals in both cases (http and https) because of http main landing page loading - before activation page.
    // Status is needed there
    setGlobals(statusInfo);

    // don't bother displaying the status if we're not on a secured channel
    if (isSecured()) {
        // Milen: For statusInfo format see StatusCodeCmd.cpp

        // stop 'secureonly' css fields from being hidden
        $('.secureonly').removeClass('secureonly');

        displayStatus(statusInfo);

        // set the domain if passive provided one (set in the admin panel)
        if (statusInfo["default-domain"]) {
            $('#user-domain').val(statusInfo["default-domain"]);
        }

        // Init global settings from parameters
        setGlobals(parseParams());

        updateContent();

        // show home page wrapper
        $('#pageHomeWrapper').css('display', "block");

        /* Info windows */
        addFocusHandler("user-name", true); // set autofocus
        addFocusHandler("user-password");
        addFocusHandler("user-domain");
        addFocusHandler("code");

    } else {
        // if we're not secured, we redirect the user to the secured version of this page
        var securedUrl = generateSecuredUrl();
        log("eventually redirecting user to " + securedUrl);

        // do not redirect the home page (page before activation page)
        //if ($("#pageHome").css('display') == "none") {
            redirectUserEventually(securedUrl, REDIRECT_SECONDS);
        //}
    }
}

// function to Disable/Enable input controls after Activate button clicked
function activeButtonOnOff(onDisable) {

    if (onDisable)
        $('#activButton').button('loading');
    else
        $('#activButton').button('reset');

    document.getElementById("user-name").disabled = onDisable;
    document.getElementById("user-password").disabled = onDisable;
    document.getElementById("user-domain").disabled = onDisable;
    document.getElementById("code").disabled = onDisable;
    document.getElementById("activButton").disabled = onDisable;
    document.getElementById("activButtonCancel").disabled = onDisable;
}

/**
 *
 * @param {Boolean} v
 * @return {String}
 */
function showBool(v) {
    if (v) {
        return getLocaleStr("true_value");
    } else {
        return getLocaleStr("false_value");
    }
}

/**
 *
 * @param {Object} statusInfo format is:
 * {
 *    "version": "2.0.0",
 *    "build": "2.0.0",
 *    "updateable": false,
 *    "debug": true,
 *    "connected": false,
 *    "thumbnails": true
 * }
 */
function displayStatus(statusInfo) {
    var verString = getLocaleStr("app_version", {ver: statusInfo.version});
    var extraInfo = getLocaleStr("tooltip_build") + ': ' + statusInfo.build;
    extraInfo += '\n' + getLocaleStr("tooltip_version") + ': ' + statusInfo.version;
    extraInfo += '\n' + getLocaleStr("tooltip_debug") + ': ' + showBool(statusInfo.debug);
    extraInfo += '\n' + getLocaleStr("tooltip_updateable") + ': ' + showBool(statusInfo.updateable);
    extraInfo += '\n' + getLocaleStr("tooltip_connected") + ': ' + showBool(statusInfo.connected);
    extraInfo += '\n' + getLocaleStr("tooltip_thumbnails") + ': ' + showBool(statusInfo.thumbnails);

    $("#version").text(verString).prop('title', extraInfo);

    if (!statusInfo.updateable) {
        log("updates are disabled!");
        showWarning(getLocaleStr("updates_disabled_body"));
    }
}

function updateContent() {

    setTextContext("l2");
    setTextContext("l14");

    setTextContext("field_username_info");
    setTextContext("field_password_info");
    setTextContext("field_domain_info");
    setTextContext("field_code_info");
    setImageContext("scrUsername");
    setImageContext("scrPassword");
    setImageContext("scrDomain");
}

function onSendLogs(event) {
    event.preventDefault();

    var code = getComp(CODE_FIELD_CLASS).val();
    // code param is obligatory
    if (code) {
        var sendLogsUrl = "/send-logs.html?code=" + encodeURIComponent(code);

        // add simple string parameters
        function addParam(paramName) {
            var paramValue = gup(paramName);
            if (paramValue) {
                sendLogsUrl += "&" + paramName + "=" + encodeURIComponent(paramValue);
            }
        }
        addParam("uid");
        addParam("env");
        addParam("type");
        addParam("device");
        addParam("alias");
        addParam("mspId");

        // treat 'meta' param differently as it should be an object
        var meta = gup("meta");
        if (meta) {
            meta = JSON.parse(meta);
        } else {
            meta = {};
        }
        meta.isActivate = true;

        sendLogsUrl += "&meta=" + encodeURIComponent(JSON.stringify(meta));

        window.open(sendLogsUrl,'_blank');


    } else {
        // TODO Rumen - show info/warning or something
        // for now do nothing
    }
}


function onRegisterClick(event, devicePlatform) {
    var aliasURL = gup("alias");
    event.preventDefault();
    var registerUrl = "https://share.fileflex.com/fbweb/app/public/view/register?type=canvas";

    //pass platform if no alias is found in url
    if (aliasURL=="") {
        registerUrl += "&alias=" + devicePlatform;
    } else {
        registerUrl += "&alias=" + aliasURL;
    }

    window.open(registerUrl,'_blank');
}

function onActivateClick(event, devicePlatform) {
    var aliasURL = gup("alias");
    event.preventDefault();
    var activateUrl = window.location.href;

    //pass platform if no alias is found in url
    if (aliasURL=="") {
        //check if if we already have a parameter in url
        if (window.location.search.indexOf('?') > -1) {
            var urlUniteString = "&";
        } else {
            var urlUniteString = "?";
        }
        activateUrl += urlUniteString + "alias=" + devicePlatform + "#activate";
    } else {
        activateUrl += "#activate";
    }

    window.open(activateUrl, '_self');
}